require 'json'
require 'uu/os/gvc/message'

module UU
  module OS
    module GVC

      # Common component object.
      class Component

        # Raw component attributes.
        # @deprecated Direct access to attribute values through the attributes attribute is deprecated and is not going to be supported in the future versions. Use individual attribute accessors instead.
        attr_reader :attributes

        # Unique component ID.
        attr_reader :id

        # Component name.
        attr_reader :name

        # Component code.
        attr_reader :code

        # Component type.
        attr_reader :component_type

        # Component width.
        #
        # @deprecated component may not respect this dimension
        attr_reader :width

        # Component height.
        #
        # @deprecated component may not respect this dimension
        attr_reader :height

        # ID of form to which component belongs.
        attr_reader :form

        # Creates new instance of component.
        #
        # @param [UU::OS::GVC::Container] container Container where to insert new component
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(container, data = nil)
          # Moved 'require' here because there was a cyclic dependency between
          # component.rb and container.rb.
          require 'uu/os/gvc/container'

          if (container.nil?)
            raise ArgumentError.new('Parent container must be defined to create new component.')
          end
          if data.nil?
            @attributes = {}
            return
          end
          if data.kind_of?String
            data = JSON.parse(data, :symbolize_names => true)
          end
          if data.kind_of?Hash
            @attributes = data
          elsif data.kind_of?Component
            if data.attributes
              @attributes = data.attributes
            else
              @attributes = {}
            end
          else
            raise ArgumentError.new("Component data must be JSON String or Hash, but was #{data.class}.")
          end
          @container = container
        end

        def id
          @attributes[:id]
        end

        def name
          @attributes[:name]
        end

        def code
          @attributes[:code]
        end

        def component_type
          @attributes[:componentType]
        end

        def width
          @attributes[:width]
        end

        def height
          @attributes[:height]
        end

        def form
          @attributes[:form]
        end

        # Returns list of all component messages.
        #
        # @return [Array<UU::OS::GVC::Message>] List of all messages
        def messages
          result = []
          @container.messages.each do |message|
            result << message if message.related_to.include?id
          end
          result
        end

        # Adds message for component.
        #
        # @param [UU::OS::GVC::Message] message Instance of message
        def add_message(message)
          msg = UU::OS::GVC::Message.new(message)
          msg.related_to << id
          @container.add_message(msg)
        end

        # Sets focus to this component. Actual behavior of focus depends
        # on type of focused component and also on type of container in
        # which is component placed.
        # Only one component can be focused, setting focus on one component
        # disables focus of previously focused component within container.
        #
        # @return [FalseClass, TrueClass] True if focus was set, else false
        def focus()
          return @container.focus(id)
        end

      end

    end
  end
end
